@@ -49,17 +49,20 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			if self.conn_url is None and url is None:
 				print('No url was set, cant do logon')
 			if url is not None:
-				self.conn_url = SMBConnectionURL(url)
+				self.conn_url = SMBConnectionURL(url)				
 			
 			self.connection  = self.conn_url.get_connection()
 			
 			logger.debug(self.conn_url.get_credential())
 			logger.debug(self.conn_url.get_target())
 
-			await self.connection.login()
+			_, err = await self.connection.login()
+			if err is not None:
+				raise err
 			self.machine = SMBMachine(self.connection)
 		except Exception as e:
 			traceback.print_exc()
+			print('Login failed! Reason: %s' % str(err))
 		else:
 			print('Login success')
 
@@ -305,11 +308,15 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			print('file not in current directory!')
 			return
 		file_obj = self.__current_directory.files[file_name]
-		sid = await file_obj.get_security_descriptor(self.connection)
+		sid, err = await file_obj.get_security_descriptor(self.connection)
+		if err is not None:
+			raise err
 		print(str(sid))
 
 	async def do_dirsid(self):
-		sid = await self.__current_directory.get_security_descriptor(self.connection)
+		sid, err = await self.__current_directory.get_security_descriptor(self.connection)
+		if err is not None:
+			raise err
 		print(str(sid))
 
 	def _cd_completions(self):
@@ -470,7 +477,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 				file_obj = self.__current_directory.files[file_name]
 				with tqdm.tqdm(desc = 'Downloading %s' % file_name, total=file_obj.size, unit='B', unit_scale=True, unit_divisor=1024) as pbar:
 					with open(file_name, 'wb') as outfile:
-						async for data in self.machine.get_file_data(file_obj):
+						async for data, err in self.machine.get_file_data(file_obj):
+							if err is not None:
+								raise err
 							if data is None:
 								break
 							outfile.write(data)
@@ -570,7 +579,82 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 		except Exception as e:
 			traceback.print_exc()
 
-			
+	async def do_tasks(self):
+		"""List scheduled tasks """
+		try:
+			async for taskname, err in await self.machine.tasks_list():
+				if err is not None:
+					raise err
+				print(taskname)
+		
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
+
+	async def do_taskregister(self, template_file, task_name = None):
+		"""Registers a new scheduled task"""
+		try:
+			with open(template_file, 'r') as f:
+				template = f.read()
+
+			res, err = await self.machine.tasks_register(template, task_name = task_name)
+			if err is not None:
+				logger.info('[!] Failed to register new task!')
+				raise err
+		
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
+
+	async def do_taskdel(self, task_name):
+		"""Deletes a scheduled task	"""
+		try:
+			await self.machine.tasks_delete(task_name)
+
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
+
+
+	async def do_taskcmdexec(self, command):
+		""" Executes a shell command using the scheduled tasks service"""
+		try:
+			await self.machine.tasks_execute_commands([command])
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
 
 async def amain(args):
 	client = SMBClient(args.smb_url)
@@ -588,6 +672,7 @@ async def amain(args):
 def main():
 	import argparse
 	import platform
+	import logging
 	
 	parser = argparse.ArgumentParser(description='Interactive SMB client')
 	parser.add_argument('-v', '--verbose', action='count', default=0)
@@ -598,6 +683,9 @@ def main():
 	args = parser.parse_args()
 	print(__banner__)
 
+	if args.verbose >=1:
+		logger.setLevel(logging.DEBUG)
+
 	if args.verbose > 2:
 		print('setting deepdebug')
 		logger.setLevel(1) #enabling deep debug
